/*
 *  This file is part of uIRC. (https://git.redxen.eu/caskd/uIRC)
 *  Copyright (c) 2019-2021 Alex-David Denes
 *
 *  uIRC is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  uIRC is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with uIRC.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "uirc/memory.h"

#include <assert.h>         // assert()
#include <corelibs/llist.h> // llist_t
#include <stddef.h>         // ptrdiff_t
#include <stdlib.h>         // free()

void*
uirc_list_append(llist_t* anchor, void* content)
{
	assert(content != NULL);
	llist_t* new = llist_elem_alloc(content);
	if (new == NULL) return NULL;
	llist_elem_conn(anchor, new);
	return new;
}

ptrdiff_t
uirc_list_free(llist_t* list, IRC_Struct_Type type)
{
	assert(list != NULL);
	ptrdiff_t cnt = 0;
	for (; list != NULL; cnt++) {
		llist_t* const save = list;
		list                = llist_elem_get_next(list);
		void* cont          = llist_elem_get_cont(save);
		if (llist_elem_get_cont(save) != NULL) uirc_struct_free(cont, type);
		free(cont);
		llist_elem_rm(save);
	}
	return cnt;
}

